'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.readFile = readFile;

var _types = require('./types');

var esprima = require('esprima-fb');
var path = require('path');
var lodash = require('lodash');
var assign = lodash.assign;
var curry = lodash.curry;

var fs = require('fs');

function readFile(filepath) {
  return findTypes(parseFile(filepath));
}

function parseFile(filepath) {
  var data = fs.readFileSync(filepath);
  return esprima.parse(data.toString(), { tolerant: true });
}

function findTypes(tree) {
  var aliases = tree.body.map(function (s) {

    if (s.type == 'ExportDeclaration') {
      var ex = s;
      s = ex.declaration;
    }

    if (s.type == 'TypeAlias') {
      return s;
    }
  });

  return aliases.reduce(function (values, alias) {
    if (alias) {
      values[alias.id.name] = toType(alias.right);
    }
    return values;
  }, {});
}

function toProperty(prop) {
  var p = {
    key: prop.key.name,
    type: toType(prop.value) };

  if (prop.optional) {
    p.optional = true;
  }

  return p;
}

function toType(anno) {

  if (anno.type === Syntax.ObjectTypeAnnotation) {
    return objectType(anno);
  } else if (anno.type === Syntax.GenericTypeAnnotation) {
    return genericType(anno);
  } else if (anno.type === Syntax.NullableTypeAnnotation) {
    return nullableType(anno);
  } else if (anno.type === Syntax.StringLiteralTypeAnnotation) {
    return literalType(anno);
  } else {
    return valueType(anno);
  }
}

function genericType(anno) {
  var type = emptyType(anno.id.name);

  if (anno.typeParameters) {
    type.params = anno.typeParameters.params.map(toType);
  }

  return type;
}

function objectType(anno) {
  var type = emptyType('Object');
  type.properties = anno.properties.map(toProperty);
  return type;
}

function nullableType(anno) {
  var type = toType(anno.typeAnnotation);
  type.nullable = true;
  return type;
}

function literalType(anno) {
  var type = valueType(anno);
  type.literal = anno.value;
  return type;
}

function valueType(anno) {
  var type = emptyType(shortName(anno));
  return type;
}

function emptyType(name) {
  return {
    name: name };
}

function shortName(anno) {

  if (anno.type === Syntax.StringTypeAnnotation) {
    return 'string';
  } else if (anno.type === Syntax.NumberTypeAnnotation) {
    return 'number';
  } else if (anno.type === Syntax.BooleanTypeAnnotation) {
    return 'boolean';
  } else if (anno.type === Syntax.AnyTypeAnnotation) {
    return 'any';
  }

  return anno.type.replace('TypeAnnotation', '');
}

var Syntax = esprima.Syntax;